/********************************************************************************
** Form generated from reading UI file 'kfilepropspluginwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KFILEPROPSPLUGINWIDGET_H
#define UI_KFILEPROPSPLUGINWIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcapacitybar.h"
#include "kiconbutton.h"
#include "kmessagewidget.h"
#include "kseparator.h"
#include "ksqueezedtextlabel.h"

QT_BEGIN_NAMESPACE

class Ui_KFilePropsPluginWidget
{
public:
    QVBoxLayout *verticalLayout_3;
    QGridLayout *gridLayout;
    QHBoxLayout *iconLayout;
    QSpacerItem *horizontalSpacer;
    KIconButton *iconButton;
    QLabel *iconLabel;
    QHBoxLayout *fileNameLayout;
    QLineEdit *fileNameLineEdit;
    QLabel *fileNameLabel;
    KSeparator *kseparator_2;
    QLabel *typeLabel;
    QVBoxLayout *mimeTypeLayout;
    QLabel *mimeCommentLabel;
    QLabel *contentLabel;
    QLabel *magicMimeCommentLabel;
    QLabel *defaultHandlerLabel_Left;
    QHBoxLayout *defaultHandlerLayout;
    QLabel *defaultHandlerIcon;
    QLabel *defaultHandlerLabel;
    QSpacerItem *horizontalSpacer_3;
    QPushButton *configureMimeBtn;
    QLabel *locationLabel_Left;
    KSqueezedTextLabel *locationLabel;
    QLabel *sizeLabelLeft;
    QLabel *sizeLabel;
    QWidget *sizeBtnWidget;
    QHBoxLayout *horizontalLayout;
    QHBoxLayout *sizeButtonsLayout;
    QPushButton *calculateSizeBtn;
    QPushButton *stopCalculateSizeBtn;
    QPushButton *sizeDetailsBtn;
    QSpacerItem *horizontalSpacer_2;
    QLabel *symlinkTargetLabel;
    QHBoxLayout *symlinkLayout;
    QLineEdit *symlinkTargetEdit;
    QPushButton *symlinkTargetOpenDir;
    KMessageWidget *symlinkTargetMessageWidget;
    QLabel *createdTimeLabel_Left;
    QLabel *createdTimeLabel;
    QLabel *modifiedTimeLabel_Left;
    QLabel *modifiedTimeLabel;
    QLabel *accessTimeLabel_Left;
    QLabel *accessTimeLabel;
    KSeparator *fsSeparator;
    QLabel *fsLabel_Left;
    QLabel *fsLabel;
    QLabel *mountPointLabel_Left;
    KSqueezedTextLabel *mountPointLabel;
    QLabel *mountSrcLabel_Left;
    QLabel *mountSrcLabel;
    QLabel *freespaceLabel;
    KCapacityBar *capacityBar;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *KFilePropsPluginWidget)
    {
        if (KFilePropsPluginWidget->objectName().isEmpty())
            KFilePropsPluginWidget->setObjectName("KFilePropsPluginWidget");
        KFilePropsPluginWidget->resize(1053, 976);
        verticalLayout_3 = new QVBoxLayout(KFilePropsPluginWidget);
        verticalLayout_3->setObjectName("verticalLayout_3");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        gridLayout->setHorizontalSpacing(24);
        gridLayout->setVerticalSpacing(10);
        iconLayout = new QHBoxLayout();
        iconLayout->setObjectName("iconLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        iconLayout->addItem(horizontalSpacer);

        iconButton = new KIconButton(KFilePropsPluginWidget);
        iconButton->setObjectName("iconButton");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(iconButton->sizePolicy().hasHeightForWidth());
        iconButton->setSizePolicy(sizePolicy);
        iconButton->setIconSize(48);

        iconLayout->addWidget(iconButton);

        iconLabel = new QLabel(KFilePropsPluginWidget);
        iconLabel->setObjectName("iconLabel");
        iconLabel->setAlignment(Qt::AlignCenter);

        iconLayout->addWidget(iconLabel);


        gridLayout->addLayout(iconLayout, 0, 0, 1, 1);

        fileNameLayout = new QHBoxLayout();
        fileNameLayout->setObjectName("fileNameLayout");
        fileNameLineEdit = new QLineEdit(KFilePropsPluginWidget);
        fileNameLineEdit->setObjectName("fileNameLineEdit");

        fileNameLayout->addWidget(fileNameLineEdit);

        fileNameLabel = new QLabel(KFilePropsPluginWidget);
        fileNameLabel->setObjectName("fileNameLabel");
        fileNameLabel->setTextFormat(Qt::PlainText);
        fileNameLabel->setTextInteractionFlags(Qt::TextBrowserInteraction|Qt::TextSelectableByKeyboard);

        fileNameLayout->addWidget(fileNameLabel);


        gridLayout->addLayout(fileNameLayout, 0, 1, 1, 1);

        kseparator_2 = new KSeparator(KFilePropsPluginWidget);
        kseparator_2->setObjectName("kseparator_2");

        gridLayout->addWidget(kseparator_2, 1, 0, 1, 2);

        typeLabel = new QLabel(KFilePropsPluginWidget);
        typeLabel->setObjectName("typeLabel");
        typeLabel->setAlignment(Qt::AlignRight|Qt::AlignTop|Qt::AlignTrailing);

        gridLayout->addWidget(typeLabel, 2, 0, 1, 1);

        mimeTypeLayout = new QVBoxLayout();
        mimeTypeLayout->setObjectName("mimeTypeLayout");
        mimeCommentLabel = new QLabel(KFilePropsPluginWidget);
        mimeCommentLabel->setObjectName("mimeCommentLabel");
        mimeCommentLabel->setTextFormat(Qt::PlainText);
        mimeCommentLabel->setTextInteractionFlags(Qt::TextBrowserInteraction|Qt::TextSelectableByKeyboard);

        mimeTypeLayout->addWidget(mimeCommentLabel);


        gridLayout->addLayout(mimeTypeLayout, 2, 1, 1, 1);

        contentLabel = new QLabel(KFilePropsPluginWidget);
        contentLabel->setObjectName("contentLabel");
        contentLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(contentLabel, 3, 0, 1, 1);

        magicMimeCommentLabel = new QLabel(KFilePropsPluginWidget);
        magicMimeCommentLabel->setObjectName("magicMimeCommentLabel");
        magicMimeCommentLabel->setTextFormat(Qt::PlainText);

        gridLayout->addWidget(magicMimeCommentLabel, 3, 1, 1, 1);

        defaultHandlerLabel_Left = new QLabel(KFilePropsPluginWidget);
        defaultHandlerLabel_Left->setObjectName("defaultHandlerLabel_Left");
        defaultHandlerLabel_Left->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(defaultHandlerLabel_Left, 4, 0, 1, 1);

        defaultHandlerLayout = new QHBoxLayout();
        defaultHandlerLayout->setObjectName("defaultHandlerLayout");
        defaultHandlerIcon = new QLabel(KFilePropsPluginWidget);
        defaultHandlerIcon->setObjectName("defaultHandlerIcon");

        defaultHandlerLayout->addWidget(defaultHandlerIcon);

        defaultHandlerLabel = new QLabel(KFilePropsPluginWidget);
        defaultHandlerLabel->setObjectName("defaultHandlerLabel");
        defaultHandlerLabel->setTextFormat(Qt::PlainText);
        defaultHandlerLabel->setTextInteractionFlags(Qt::TextBrowserInteraction|Qt::TextSelectableByKeyboard);

        defaultHandlerLayout->addWidget(defaultHandlerLabel);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        defaultHandlerLayout->addItem(horizontalSpacer_3);

        configureMimeBtn = new QPushButton(KFilePropsPluginWidget);
        configureMimeBtn->setObjectName("configureMimeBtn");
        sizePolicy.setHeightForWidth(configureMimeBtn->sizePolicy().hasHeightForWidth());
        configureMimeBtn->setSizePolicy(sizePolicy);
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("configure")));
        configureMimeBtn->setIcon(icon);

        defaultHandlerLayout->addWidget(configureMimeBtn);


        gridLayout->addLayout(defaultHandlerLayout, 4, 1, 1, 1);

        locationLabel_Left = new QLabel(KFilePropsPluginWidget);
        locationLabel_Left->setObjectName("locationLabel_Left");
        locationLabel_Left->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(locationLabel_Left, 5, 0, 1, 1);

        locationLabel = new KSqueezedTextLabel(KFilePropsPluginWidget);
        locationLabel->setObjectName("locationLabel");
        locationLabel->setLayoutDirection(Qt::LeftToRight);
        locationLabel->setTextFormat(Qt::PlainText);
        locationLabel->setTextInteractionFlags(Qt::TextBrowserInteraction|Qt::TextSelectableByKeyboard);

        gridLayout->addWidget(locationLabel, 5, 1, 1, 1);

        sizeLabelLeft = new QLabel(KFilePropsPluginWidget);
        sizeLabelLeft->setObjectName("sizeLabelLeft");
        sizeLabelLeft->setAlignment(Qt::AlignRight|Qt::AlignTop|Qt::AlignTrailing);

        gridLayout->addWidget(sizeLabelLeft, 6, 0, 1, 1);

        sizeLabel = new QLabel(KFilePropsPluginWidget);
        sizeLabel->setObjectName("sizeLabel");
        sizeLabel->setTextFormat(Qt::PlainText);
        sizeLabel->setTextInteractionFlags(Qt::TextBrowserInteraction|Qt::TextSelectableByKeyboard);

        gridLayout->addWidget(sizeLabel, 6, 1, 1, 1);

        sizeBtnWidget = new QWidget(KFilePropsPluginWidget);
        sizeBtnWidget->setObjectName("sizeBtnWidget");
        horizontalLayout = new QHBoxLayout(sizeBtnWidget);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        sizeButtonsLayout = new QHBoxLayout();
        sizeButtonsLayout->setObjectName("sizeButtonsLayout");
        calculateSizeBtn = new QPushButton(sizeBtnWidget);
        calculateSizeBtn->setObjectName("calculateSizeBtn");
        QIcon icon1(QIcon::fromTheme(QString::fromUtf8("view-refresh")));
        calculateSizeBtn->setIcon(icon1);

        sizeButtonsLayout->addWidget(calculateSizeBtn);

        stopCalculateSizeBtn = new QPushButton(sizeBtnWidget);
        stopCalculateSizeBtn->setObjectName("stopCalculateSizeBtn");
        QIcon icon2(QIcon::fromTheme(QString::fromUtf8("dialog-cancel")));
        stopCalculateSizeBtn->setIcon(icon2);

        sizeButtonsLayout->addWidget(stopCalculateSizeBtn);

        sizeDetailsBtn = new QPushButton(sizeBtnWidget);
        sizeDetailsBtn->setObjectName("sizeDetailsBtn");
        QIcon icon3(QIcon::fromTheme(QString::fromUtf8("filelight")));
        sizeDetailsBtn->setIcon(icon3);

        sizeButtonsLayout->addWidget(sizeDetailsBtn);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        sizeButtonsLayout->addItem(horizontalSpacer_2);


        horizontalLayout->addLayout(sizeButtonsLayout);


        gridLayout->addWidget(sizeBtnWidget, 7, 1, 1, 1);

        symlinkTargetLabel = new QLabel(KFilePropsPluginWidget);
        symlinkTargetLabel->setObjectName("symlinkTargetLabel");
        symlinkTargetLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(symlinkTargetLabel, 8, 0, 1, 1);

        symlinkLayout = new QHBoxLayout();
        symlinkLayout->setObjectName("symlinkLayout");
        symlinkTargetEdit = new QLineEdit(KFilePropsPluginWidget);
        symlinkTargetEdit->setObjectName("symlinkTargetEdit");

        symlinkLayout->addWidget(symlinkTargetEdit);

        symlinkTargetOpenDir = new QPushButton(KFilePropsPluginWidget);
        symlinkTargetOpenDir->setObjectName("symlinkTargetOpenDir");
        sizePolicy.setHeightForWidth(symlinkTargetOpenDir->sizePolicy().hasHeightForWidth());
        symlinkTargetOpenDir->setSizePolicy(sizePolicy);
        QIcon icon4(QIcon::fromTheme(QString::fromUtf8("go-jump")));
        symlinkTargetOpenDir->setIcon(icon4);

        symlinkLayout->addWidget(symlinkTargetOpenDir);


        gridLayout->addLayout(symlinkLayout, 8, 1, 1, 1);

        symlinkTargetMessageWidget = new KMessageWidget(KFilePropsPluginWidget);
        symlinkTargetMessageWidget->setObjectName("symlinkTargetMessageWidget");
        symlinkTargetMessageWidget->setWordWrap(true);
        symlinkTargetMessageWidget->setMessageType(KMessageWidget::Error);

        gridLayout->addWidget(symlinkTargetMessageWidget, 9, 0, 1, 2);

        createdTimeLabel_Left = new QLabel(KFilePropsPluginWidget);
        createdTimeLabel_Left->setObjectName("createdTimeLabel_Left");
        createdTimeLabel_Left->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(createdTimeLabel_Left, 10, 0, 1, 1);

        createdTimeLabel = new QLabel(KFilePropsPluginWidget);
        createdTimeLabel->setObjectName("createdTimeLabel");
        createdTimeLabel->setTextInteractionFlags(Qt::TextBrowserInteraction|Qt::TextSelectableByKeyboard);

        gridLayout->addWidget(createdTimeLabel, 10, 1, 1, 1);

        modifiedTimeLabel_Left = new QLabel(KFilePropsPluginWidget);
        modifiedTimeLabel_Left->setObjectName("modifiedTimeLabel_Left");
        modifiedTimeLabel_Left->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(modifiedTimeLabel_Left, 11, 0, 1, 1);

        modifiedTimeLabel = new QLabel(KFilePropsPluginWidget);
        modifiedTimeLabel->setObjectName("modifiedTimeLabel");
        modifiedTimeLabel->setTextInteractionFlags(Qt::TextBrowserInteraction|Qt::TextSelectableByKeyboard);

        gridLayout->addWidget(modifiedTimeLabel, 11, 1, 1, 1);

        accessTimeLabel_Left = new QLabel(KFilePropsPluginWidget);
        accessTimeLabel_Left->setObjectName("accessTimeLabel_Left");
        accessTimeLabel_Left->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(accessTimeLabel_Left, 12, 0, 1, 1);

        accessTimeLabel = new QLabel(KFilePropsPluginWidget);
        accessTimeLabel->setObjectName("accessTimeLabel");
        accessTimeLabel->setTextInteractionFlags(Qt::TextBrowserInteraction|Qt::TextSelectableByKeyboard);

        gridLayout->addWidget(accessTimeLabel, 12, 1, 1, 1);

        fsSeparator = new KSeparator(KFilePropsPluginWidget);
        fsSeparator->setObjectName("fsSeparator");

        gridLayout->addWidget(fsSeparator, 13, 0, 1, 2);

        fsLabel_Left = new QLabel(KFilePropsPluginWidget);
        fsLabel_Left->setObjectName("fsLabel_Left");
        fsLabel_Left->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(fsLabel_Left, 14, 0, 1, 1);

        fsLabel = new QLabel(KFilePropsPluginWidget);
        fsLabel->setObjectName("fsLabel");
        fsLabel->setTextFormat(Qt::PlainText);
        fsLabel->setTextInteractionFlags(Qt::TextBrowserInteraction|Qt::TextSelectableByKeyboard);

        gridLayout->addWidget(fsLabel, 14, 1, 1, 1);

        mountPointLabel_Left = new QLabel(KFilePropsPluginWidget);
        mountPointLabel_Left->setObjectName("mountPointLabel_Left");
        mountPointLabel_Left->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(mountPointLabel_Left, 15, 0, 1, 1);

        mountPointLabel = new KSqueezedTextLabel(KFilePropsPluginWidget);
        mountPointLabel->setObjectName("mountPointLabel");
        mountPointLabel->setTextFormat(Qt::PlainText);
        mountPointLabel->setTextInteractionFlags(Qt::TextBrowserInteraction|Qt::TextSelectableByKeyboard);

        gridLayout->addWidget(mountPointLabel, 15, 1, 1, 1);

        mountSrcLabel_Left = new QLabel(KFilePropsPluginWidget);
        mountSrcLabel_Left->setObjectName("mountSrcLabel_Left");
        mountSrcLabel_Left->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(mountSrcLabel_Left, 16, 0, 1, 1);

        mountSrcLabel = new QLabel(KFilePropsPluginWidget);
        mountSrcLabel->setObjectName("mountSrcLabel");
        mountSrcLabel->setTextFormat(Qt::PlainText);
        mountSrcLabel->setTextInteractionFlags(Qt::TextBrowserInteraction|Qt::TextSelectableByKeyboard);

        gridLayout->addWidget(mountSrcLabel, 16, 1, 1, 1);

        freespaceLabel = new QLabel(KFilePropsPluginWidget);
        freespaceLabel->setObjectName("freespaceLabel");
        freespaceLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(freespaceLabel, 17, 0, 1, 1);

        capacityBar = new KCapacityBar(KFilePropsPluginWidget);
        capacityBar->setObjectName("capacityBar");
        capacityBar->setDrawTextMode(KCapacityBar::DrawTextOutline);

        gridLayout->addWidget(capacityBar, 17, 1, 1, 1);

        gridLayout->setColumnStretch(1, 1);

        verticalLayout_3->addLayout(gridLayout);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);

#if QT_CONFIG(shortcut)
        typeLabel->setBuddy(mimeCommentLabel);
        defaultHandlerLabel_Left->setBuddy(defaultHandlerLabel);
        locationLabel_Left->setBuddy(locationLabel);
        sizeLabelLeft->setBuddy(sizeLabel);
        symlinkTargetLabel->setBuddy(symlinkTargetEdit);
        createdTimeLabel_Left->setBuddy(createdTimeLabel);
        modifiedTimeLabel_Left->setBuddy(modifiedTimeLabel);
        accessTimeLabel_Left->setBuddy(accessTimeLabel);
        fsLabel_Left->setBuddy(fsLabel);
        mountPointLabel_Left->setBuddy(mountPointLabel);
        mountSrcLabel_Left->setBuddy(mountSrcLabel);
        freespaceLabel->setBuddy(capacityBar);
#endif // QT_CONFIG(shortcut)

        retranslateUi(KFilePropsPluginWidget);

        QMetaObject::connectSlotsByName(KFilePropsPluginWidget);
    } // setupUi

    void retranslateUi(QWidget *KFilePropsPluginWidget)
    {
        iconLabel->setText(QString());
        fileNameLabel->setText(QString());
        typeLabel->setText(tr2i18n("Type:", nullptr));
        mimeCommentLabel->setText(QString());
        contentLabel->setText(tr2i18n("Content:", nullptr));
        magicMimeCommentLabel->setText(QString());
        defaultHandlerLabel_Left->setText(tr2i18n("Open with:", nullptr));
        defaultHandlerIcon->setText(QString());
        defaultHandlerLabel->setText(QString());
        configureMimeBtn->setText(tr2i18n("Change...", nullptr));
        locationLabel_Left->setText(tr2i18n("Location:", nullptr));
        locationLabel->setText(QString());
        sizeLabelLeft->setText(tr2i18n("Size:", nullptr));
        sizeLabel->setText(QString());
        calculateSizeBtn->setText(tr2i18n("Calculate", nullptr));
#if QT_CONFIG(tooltip)
        calculateSizeBtn->setToolTip(tr2i18n("Determine the folder size", nullptr));
#endif // QT_CONFIG(tooltip)
        stopCalculateSizeBtn->setText(tr2i18n("Stop", nullptr));
#if QT_CONFIG(tooltip)
        stopCalculateSizeBtn->setToolTip(tr2i18n("Stop determining the folder size", nullptr));
#endif // QT_CONFIG(tooltip)
        sizeDetailsBtn->setText(tr2i18n("Explore in Filelight", nullptr));
        symlinkTargetLabel->setText(tr2i18n("Points to:", nullptr));
        symlinkTargetEdit->setText(QString());
        symlinkTargetOpenDir->setText(QString());
#if QT_CONFIG(accessibility)
        symlinkTargetOpenDir->setAccessibleName(tr2i18n("Show target", nullptr));
#endif // QT_CONFIG(accessibility)
        createdTimeLabel_Left->setText(tr2i18n("Created:", nullptr));
        createdTimeLabel->setText(QString());
        modifiedTimeLabel_Left->setText(tr2i18n("Modified:", nullptr));
        modifiedTimeLabel->setText(QString());
        accessTimeLabel_Left->setText(tr2i18n("Accessed:", nullptr));
        accessTimeLabel->setText(QString());
        fsLabel_Left->setText(tr2i18n("File system:", nullptr));
        fsLabel->setText(QString());
        mountPointLabel_Left->setText(tr2i18n("Mounted on:", nullptr));
        mountPointLabel->setText(QString());
        mountSrcLabel_Left->setText(tr2i18n("Mounted from:", nullptr));
        mountSrcLabel->setText(QString());
        freespaceLabel->setText(tr2i18n("Free space:", nullptr));
        capacityBar->setText(tr2i18n("Unknown size", nullptr));
        (void)KFilePropsPluginWidget;
    } // retranslateUi

};

namespace Ui {
    class KFilePropsPluginWidget: public Ui_KFilePropsPluginWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KFILEPROPSPLUGINWIDGET_H

