/****************************************************************************
** Meta object code from reading C++ file 'managerinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "managerinterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'managerinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN38OrgFreedesktopSystemd1ManagerInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgFreedesktopSystemd1ManagerInterface::qt_create_metaobjectdata<qt_meta_tag_ZN38OrgFreedesktopSystemd1ManagerInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgFreedesktopSystemd1ManagerInterface",
        "JobRemoved",
        "",
        "id",
        "QDBusObjectPath",
        "job",
        "unit",
        "result",
        "UnitNew",
        "name",
        "path",
        "StartTransientUnit",
        "QDBusPendingReply<>",
        "mode",
        "QVariantMultiMap",
        "properties",
        "TransientAuxList",
        "aux",
        "Subscribe",
        "Unsubscribe"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'JobRemoved'
        QtMocHelpers::SignalData<void(uint, const QDBusObjectPath &, const QString &, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 3 }, { 0x80000000 | 4, 5 }, { QMetaType::QString, 6 }, { QMetaType::QString, 7 },
        }}),
        // Signal 'UnitNew'
        QtMocHelpers::SignalData<void(const QString &, const QDBusObjectPath &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 }, { 0x80000000 | 4, 10 },
        }}),
        // Slot 'StartTransientUnit'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &, const QString &, const QVariantMultiMap &, TransientAuxList)>(11, 2, QMC::AccessPublic, 0x80000000 | 12, {{
            { QMetaType::QString, 9 }, { QMetaType::QString, 13 }, { 0x80000000 | 14, 15 }, { 0x80000000 | 16, 17 },
        }}),
        // Slot 'Subscribe'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(18, 2, QMC::AccessPublic, 0x80000000 | 12),
        // Slot 'Unsubscribe'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(19, 2, QMC::AccessPublic, 0x80000000 | 12),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OrgFreedesktopSystemd1ManagerInterface, qt_meta_tag_ZN38OrgFreedesktopSystemd1ManagerInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OrgFreedesktopSystemd1ManagerInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN38OrgFreedesktopSystemd1ManagerInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN38OrgFreedesktopSystemd1ManagerInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN38OrgFreedesktopSystemd1ManagerInterfaceE_t>.metaTypes,
    nullptr
} };

void OrgFreedesktopSystemd1ManagerInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgFreedesktopSystemd1ManagerInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->JobRemoved((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 1: _t->UnitNew((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[2]))); break;
        case 2: { QDBusPendingReply<> _r = _t->StartTransientUnit((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariantMultiMap>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<TransientAuxList>>(_a[4])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDBusPendingReply<> _r = _t->Subscribe();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 4: { QDBusPendingReply<> _r = _t->Unsubscribe();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QVariantMultiMap >(); break;
            case 3:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TransientAuxList >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopSystemd1ManagerInterface::*)(uint , const QDBusObjectPath & , const QString & , const QString & )>(_a, &OrgFreedesktopSystemd1ManagerInterface::JobRemoved, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopSystemd1ManagerInterface::*)(const QString & , const QDBusObjectPath & )>(_a, &OrgFreedesktopSystemd1ManagerInterface::UnitNew, 1))
            return;
    }
}

const QMetaObject *OrgFreedesktopSystemd1ManagerInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgFreedesktopSystemd1ManagerInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN38OrgFreedesktopSystemd1ManagerInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgFreedesktopSystemd1ManagerInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void OrgFreedesktopSystemd1ManagerInterface::JobRemoved(uint _t1, const QDBusObjectPath & _t2, const QString & _t3, const QString & _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 1
void OrgFreedesktopSystemd1ManagerInterface::UnitNew(const QString & _t1, const QDBusObjectPath & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}
QT_WARNING_POP
