// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "kfilefiltercombo_debug.h"


Q_LOGGING_CATEGORY(KIO_KFILEWIDGETS_KFILEFILTERCOMBO, "kf.kio.filewidgets.kfilefiltercombo", QtInfoMsg)

