/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.kio.FileUndoManager.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef FILEUNDOMANAGER_ADAPTOR_H
#define FILEUNDOMANAGER_ADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "fileundomanager_p.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.kde.libkonq.FileUndoManager
 */
class KIOFileUndoManagerAdaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.kde.libkonq.FileUndoManager")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.kde.libkonq.FileUndoManager\">\n"
"    <method name=\"get\">\n"
"      <arg direction=\"out\" type=\"ay\" name=\"commands\"/>\n"
"    </method>\n"
"    <signal name=\"lock\"/>\n"
"    <signal name=\"pop\"/>\n"
"    <signal name=\"push\">\n"
"      <arg direction=\"out\" type=\"ay\" name=\"command\"/>\n"
"    </signal>\n"
"    <signal name=\"unlock\"/>\n"
"  </interface>\n"
        "")
public:
    KIOFileUndoManagerAdaptor(KIO::FileUndoManagerPrivate *parent);
    ~KIOFileUndoManagerAdaptor() override;

    inline KIO::FileUndoManagerPrivate *parent() const
    { return static_cast<KIO::FileUndoManagerPrivate *>(QObject::parent()); }

public: // PROPERTIES
public Q_SLOTS: // METHODS
    QByteArray get();
Q_SIGNALS: // SIGNALS
    void lock();
    void pop();
    void push(const QByteArray &command);
    void unlock();
};

#endif
