/****************************************************************************
** Meta object code from reading C++ file 'kpasswdserveradaptor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "kpasswdserveradaptor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kpasswdserveradaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20KPasswdServerAdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto KPasswdServerAdaptor::qt_create_metaobjectdata<qt_meta_tag_ZN20KPasswdServerAdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPasswdServerAdaptor",
        "D-Bus Interface",
        "org.kde.KPasswdServer",
        "D-Bus Introspection",
        "  <interface name=\"org.kde.KPasswdServer\">\n    <signal name=\"c"
        "heckAuthInfoAsyncResult\">\n      <arg type=\"x\" name=\"requestId"
        "\"/>\n      <arg type=\"x\" name=\"seqNr\"/>\n      <arg type=\"(y"
        "sssssssssbbbba{s(siv)})\" name=\"info\"/>\n      <annotation value"
        "=\"KIO::AuthInfo\" name=\"org.qtproject.QtDBus.QtTypeName.Out2\"/>"
        "\n    </signal>\n    <signal name=\"queryAuthInfoAsyncResult\">\n "
        "     <arg type=\"x\" name=\"requestId\"/>\n      <arg type=\"x\" n"
        "ame=\"seqNr\"/>\n      <arg type=\"(ysssssssssbbbba{s(siv)})\" nam"
        "e=\"info\"/>\n      <annotation value=\"KIO::AuthInfo\" name=\"org"
        ".qtproject.QtDBus.QtTypeName.Out2\"/>\n    </signal>\n    <method "
        "name=\"checkAuthInfo\">\n      <arg direction=\"out\" type=\"ay\"/"
        ">\n      <arg direction=\"in\" type=\"ay\" name=\"data\"/>\n      "
        "<arg direction=\"in\" type=\"x\" name=\"windowId\"/>\n      <arg d"
        "irection=\"in\" type=\"x\" name=\"usertime\"/>\n    </method>\n   "
        " <method name=\"queryAuthInfo\">\n      <arg direction=\"out\" typ"
        "e=\"ay\"/>\n      <arg direction=\"in\" type=\"ay\" name=\"data\"/"
        ">\n      <arg direction=\"in\" type=\"s\" name=\"errorMsg\"/>\n   "
        "   <arg direction=\"in\" type=\"x\" name=\"windowId\"/>\n      <ar"
        "g direction=\"in\" type=\"x\" name=\"seqNr\"/>\n      <arg directi"
        "on=\"in\" type=\"x\" name=\"usertime\"/>\n    </method>\n    <meth"
        "od name=\"checkAuthInfoAsync\">\n      <arg direction=\"out\" type"
        "=\"x\"/>\n      <arg direction=\"in\" type=\"(ysssssssssbbbba{s(si"
        "v)})\" name=\"info\"/>\n      <annotation value=\"KIO::AuthInfo\" "
        "name=\"org.qtproject.QtDBus.QtTypeName.In0\"/>\n      <arg directi"
        "on=\"in\" type=\"x\" name=\"windowId\"/>\n      <arg direction=\"i"
        "n\" type=\"x\" name=\"usertime\"/>\n    </method>\n    <method nam"
        "e=\"queryAuthInfoAsync\">\n      <arg direction=\"out\" type=\"x\""
        "/>\n      <arg direction=\"in\" type=\"(ysssssssssbbbba{s(siv)})\""
        " name=\"info\"/>\n      <annotation value=\"KIO::AuthInfo\" name=\""
        "org.qtproject.QtDBus.QtTypeName.In0\"/>\n      <arg direction=\"in"
        "\" type=\"s\" name=\"errorMsg\"/>\n      <arg direction=\"in\" typ"
        "e=\"x\" name=\"windowId\"/>\n      <arg direction=\"in\" type=\"x\""
        " name=\"seqNr\"/>\n      <arg direction=\"in\" type=\"x\" name=\"u"
        "sertime\"/>\n    </method>\n    <method name=\"addAuthInfo\">\n   "
        "   <arg direction=\"in\" type=\"ay\" name=\"data\"/>\n      <arg d"
        "irection=\"in\" type=\"x\" name=\"windowId\"/>\n    </method>\n   "
        " <method name=\"addAuthInfo\">\n      <arg direction=\"in\" type=\""
        "(ysssssssssbbbba{s(siv)})\" name=\"info\"/>\n      <annotation val"
        "ue=\"KIO::AuthInfo\" name=\"org.qtproject.QtDBus.QtTypeName.In0\"/"
        ">\n      <arg direction=\"in\" type=\"x\" name=\"windowId\"/>\n   "
        " </method>\n    <method name=\"removeAuthInfo\">\n      <arg direc"
        "tion=\"in\" type=\"s\" name=\"host\"/>\n      <arg direction=\"in\""
        " type=\"s\" name=\"protocol\"/>\n      <arg direction=\"in\" type="
        "\"s\" name=\"user\"/>\n    </method>\n  </interface>\n",
        "checkAuthInfoAsyncResult",
        "",
        "requestId",
        "seqNr",
        "KIO::AuthInfo",
        "info",
        "queryAuthInfoAsyncResult",
        "addAuthInfo",
        "windowId",
        "data",
        "checkAuthInfo",
        "usertime",
        "checkAuthInfoAsync",
        "queryAuthInfo",
        "errorMsg",
        "queryAuthInfoAsync",
        "removeAuthInfo",
        "host",
        "protocol",
        "user"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'checkAuthInfoAsyncResult'
        QtMocHelpers::SignalData<void(qlonglong, qlonglong, KIO::AuthInfo)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 7 }, { QMetaType::LongLong, 8 }, { 0x80000000 | 9, 10 },
        }}),
        // Signal 'queryAuthInfoAsyncResult'
        QtMocHelpers::SignalData<void(qlonglong, qlonglong, KIO::AuthInfo)>(11, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 7 }, { QMetaType::LongLong, 8 }, { 0x80000000 | 9, 10 },
        }}),
        // Slot 'addAuthInfo'
        QtMocHelpers::SlotData<void(KIO::AuthInfo, qlonglong)>(12, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::LongLong, 13 },
        }}),
        // Slot 'addAuthInfo'
        QtMocHelpers::SlotData<void(const QByteArray &, qlonglong)>(12, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 14 }, { QMetaType::LongLong, 13 },
        }}),
        // Slot 'checkAuthInfo'
        QtMocHelpers::SlotData<QByteArray(const QByteArray &, qlonglong, qlonglong)>(15, 6, QMC::AccessPublic, QMetaType::QByteArray, {{
            { QMetaType::QByteArray, 14 }, { QMetaType::LongLong, 13 }, { QMetaType::LongLong, 16 },
        }}),
        // Slot 'checkAuthInfoAsync'
        QtMocHelpers::SlotData<qlonglong(KIO::AuthInfo, qlonglong, qlonglong)>(17, 6, QMC::AccessPublic, QMetaType::LongLong, {{
            { 0x80000000 | 9, 10 }, { QMetaType::LongLong, 13 }, { QMetaType::LongLong, 16 },
        }}),
        // Slot 'queryAuthInfo'
        QtMocHelpers::SlotData<QByteArray(const QByteArray &, const QString &, qlonglong, qlonglong, qlonglong)>(18, 6, QMC::AccessPublic, QMetaType::QByteArray, {{
            { QMetaType::QByteArray, 14 }, { QMetaType::QString, 19 }, { QMetaType::LongLong, 13 }, { QMetaType::LongLong, 8 },
            { QMetaType::LongLong, 16 },
        }}),
        // Slot 'queryAuthInfoAsync'
        QtMocHelpers::SlotData<qlonglong(KIO::AuthInfo, const QString &, qlonglong, qlonglong, qlonglong)>(20, 6, QMC::AccessPublic, QMetaType::LongLong, {{
            { 0x80000000 | 9, 10 }, { QMetaType::QString, 19 }, { QMetaType::LongLong, 13 }, { QMetaType::LongLong, 8 },
            { QMetaType::LongLong, 16 },
        }}),
        // Slot 'removeAuthInfo'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &)>(21, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 22 }, { QMetaType::QString, 23 }, { QMetaType::QString, 24 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<KPasswdServerAdaptor, qt_meta_tag_ZN20KPasswdServerAdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject KPasswdServerAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20KPasswdServerAdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20KPasswdServerAdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20KPasswdServerAdaptorE_t>.metaTypes,
    nullptr
} };

void KPasswdServerAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KPasswdServerAdaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->checkAuthInfoAsyncResult((*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KIO::AuthInfo>>(_a[3]))); break;
        case 1: _t->queryAuthInfoAsyncResult((*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KIO::AuthInfo>>(_a[3]))); break;
        case 2: _t->addAuthInfo((*reinterpret_cast<std::add_pointer_t<KIO::AuthInfo>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[2]))); break;
        case 3: _t->addAuthInfo((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[2]))); break;
        case 4: { QByteArray _r = _t->checkAuthInfo((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QByteArray*>(_a[0]) = std::move(_r); }  break;
        case 5: { qlonglong _r = _t->checkAuthInfoAsync((*reinterpret_cast<std::add_pointer_t<KIO::AuthInfo>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[3])));
            if (_a[0]) *reinterpret_cast<qlonglong*>(_a[0]) = std::move(_r); }  break;
        case 6: { QByteArray _r = _t->queryAuthInfo((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[5])));
            if (_a[0]) *reinterpret_cast<QByteArray*>(_a[0]) = std::move(_r); }  break;
        case 7: { qlonglong _r = _t->queryAuthInfoAsync((*reinterpret_cast<std::add_pointer_t<KIO::AuthInfo>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[5])));
            if (_a[0]) *reinterpret_cast<qlonglong*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->removeAuthInfo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KPasswdServerAdaptor::*)(qlonglong , qlonglong , KIO::AuthInfo )>(_a, &KPasswdServerAdaptor::checkAuthInfoAsyncResult, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KPasswdServerAdaptor::*)(qlonglong , qlonglong , KIO::AuthInfo )>(_a, &KPasswdServerAdaptor::queryAuthInfoAsyncResult, 1))
            return;
    }
}

const QMetaObject *KPasswdServerAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPasswdServerAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20KPasswdServerAdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int KPasswdServerAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void KPasswdServerAdaptor::checkAuthInfoAsyncResult(qlonglong _t1, qlonglong _t2, KIO::AuthInfo _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void KPasswdServerAdaptor::queryAuthInfoAsyncResult(qlonglong _t1, qlonglong _t2, KIO::AuthInfo _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
