/****************************************************************************
** Meta object code from reading C++ file 'askuseractioninterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kio-6.29.0/src/core/askuseractioninterface.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'askuseractioninterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3KIO22AskUserActionInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto KIO::AskUserActionInterface::qt_create_metaobjectdata<qt_meta_tag_ZN3KIO22AskUserActionInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KIO::AskUserActionInterface",
        "askUserRenameResult",
        "",
        "KIO::RenameDialog_Result",
        "result",
        "QUrl",
        "newUrl",
        "KJob*",
        "parentJob",
        "askUserSkipResult",
        "KIO::SkipDialog_Result",
        "askUserDeleteResult",
        "allowDelete",
        "QList<QUrl>",
        "urls",
        "KIO::AskUserActionInterface::DeletionType",
        "deletionType",
        "QWidget*",
        "parent",
        "messageBoxResult",
        "askIgnoreSslErrorsResult"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'askUserRenameResult'
        QtMocHelpers::SignalData<void(KIO::RenameDialog_Result, const QUrl &, KJob *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'askUserSkipResult'
        QtMocHelpers::SignalData<void(KIO::SkipDialog_Result, KJob *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 4 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'askUserDeleteResult'
        QtMocHelpers::SignalData<void(bool, const QList<QUrl> &, KIO::AskUserActionInterface::DeletionType, QWidget *)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 12 }, { 0x80000000 | 13, 14 }, { 0x80000000 | 15, 16 }, { 0x80000000 | 17, 18 },
        }}),
        // Signal 'messageBoxResult'
        QtMocHelpers::SignalData<void(int)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Signal 'askIgnoreSslErrorsResult'
        QtMocHelpers::SignalData<void(int)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AskUserActionInterface, qt_meta_tag_ZN3KIO22AskUserActionInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KIO::AskUserActionInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KIO22AskUserActionInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KIO22AskUserActionInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3KIO22AskUserActionInterfaceE_t>.metaTypes,
    nullptr
} };

void KIO::AskUserActionInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AskUserActionInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->askUserRenameResult((*reinterpret_cast<std::add_pointer_t<KIO::RenameDialog_Result>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[3]))); break;
        case 1: _t->askUserSkipResult((*reinterpret_cast<std::add_pointer_t<KIO::SkipDialog_Result>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[2]))); break;
        case 2: _t->askUserDeleteResult((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KIO::AskUserActionInterface::DeletionType>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[4]))); break;
        case 3: _t->messageBoxResult((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->askIgnoreSslErrorsResult((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KJob* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KJob* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AskUserActionInterface::*)(KIO::RenameDialog_Result , const QUrl & , KJob * )>(_a, &AskUserActionInterface::askUserRenameResult, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AskUserActionInterface::*)(KIO::SkipDialog_Result , KJob * )>(_a, &AskUserActionInterface::askUserSkipResult, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AskUserActionInterface::*)(bool , const QList<QUrl> & , KIO::AskUserActionInterface::DeletionType , QWidget * )>(_a, &AskUserActionInterface::askUserDeleteResult, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AskUserActionInterface::*)(int )>(_a, &AskUserActionInterface::messageBoxResult, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AskUserActionInterface::*)(int )>(_a, &AskUserActionInterface::askIgnoreSslErrorsResult, 4))
            return;
    }
}

const QMetaObject *KIO::AskUserActionInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KIO::AskUserActionInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KIO22AskUserActionInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KIO::AskUserActionInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void KIO::AskUserActionInterface::askUserRenameResult(KIO::RenameDialog_Result _t1, const QUrl & _t2, KJob * _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void KIO::AskUserActionInterface::askUserSkipResult(KIO::SkipDialog_Result _t1, KJob * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void KIO::AskUserActionInterface::askUserDeleteResult(bool _t1, const QList<QUrl> & _t2, KIO::AskUserActionInterface::DeletionType _t3, QWidget * _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 3
void KIO::AskUserActionInterface::messageBoxResult(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void KIO::AskUserActionInterface::askIgnoreSslErrorsResult(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
