/****************************************************************************
** Meta object code from reading C++ file 'kfileplacesmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kio-6.29.0/src/filewidgets/kfileplacesmodel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kfileplacesmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16KFilePlacesModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KFilePlacesModel::qt_create_metaobjectdata<qt_meta_tag_ZN16KFilePlacesModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KFilePlacesModel",
        "errorMessage",
        "",
        "message",
        "setupDone",
        "QModelIndex",
        "index",
        "success",
        "teardownDone",
        "Solid::ErrorType",
        "error",
        "QVariant",
        "errorData",
        "groupHiddenChanged",
        "KFilePlacesModel::GroupType",
        "group",
        "hidden",
        "reloaded",
        "supportedSchemesChanged",
        "url",
        "QUrl",
        "setupNeeded",
        "isTeardownAllowed",
        "isEjectAllowed",
        "isTeardownOverlayRecommended",
        "deviceAccessibility",
        "KFilePlacesModel::DeviceAccessibility",
        "icon",
        "QIcon",
        "text",
        "isHidden",
        "isGroupHidden",
        "GroupType",
        "type",
        "isDevice",
        "groupType",
        "groupIndexes",
        "QModelIndexList",
        "teardownActionForIndex",
        "QAction*",
        "openRecentOption",
        "ejectActionForIndex",
        "partitionActionForIndex",
        "requestTeardown",
        "requestEject",
        "requestSetup",
        "addPlace",
        "iconName",
        "appName",
        "after",
        "editPlace",
        "removePlace",
        "setPlaceHidden",
        "setGroupHidden",
        "movePlace",
        "itemRow",
        "row",
        "hiddenCount",
        "refresh",
        "supportedSchemes",
        "PlacesType",
        "RemoteType",
        "RecentlySavedType",
        "SearchForType",
        "DevicesType",
        "RemovableDevicesType",
        "UnknownType",
        "TagsType",
        "DeviceAccessibility",
        "SetupNeeded",
        "SetupInProgress",
        "Accessible",
        "TeardownInProgress"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'errorMessage'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'setupDone'
        QtMocHelpers::SignalData<void(const QModelIndex &, bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { QMetaType::Bool, 7 },
        }}),
        // Signal 'teardownDone'
        QtMocHelpers::SignalData<void(const QModelIndex &, Solid::ErrorType, const QVariant &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 9, 10 }, { 0x80000000 | 11, 12 },
        }}),
        // Signal 'groupHiddenChanged'
        QtMocHelpers::SignalData<void(KFilePlacesModel::GroupType, bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 }, { QMetaType::Bool, 16 },
        }}),
        // Signal 'reloaded'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'supportedSchemesChanged'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'url'
        QtMocHelpers::MethodData<QUrl(const QModelIndex &) const>(19, 2, QMC::AccessPublic, 0x80000000 | 20, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Method 'setupNeeded'
        QtMocHelpers::MethodData<bool(const QModelIndex &) const>(21, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Method 'isTeardownAllowed'
        QtMocHelpers::MethodData<bool(const QModelIndex &) const>(22, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Method 'isEjectAllowed'
        QtMocHelpers::MethodData<bool(const QModelIndex &) const>(23, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Method 'isTeardownOverlayRecommended'
        QtMocHelpers::MethodData<bool(const QModelIndex &) const>(24, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Method 'deviceAccessibility'
        QtMocHelpers::MethodData<KFilePlacesModel::DeviceAccessibility(const QModelIndex &) const>(25, 2, QMC::AccessPublic, 0x80000000 | 26, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Method 'icon'
        QtMocHelpers::MethodData<QIcon(const QModelIndex &) const>(27, 2, QMC::AccessPublic, 0x80000000 | 28, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Method 'text'
        QtMocHelpers::MethodData<QString(const QModelIndex &) const>(29, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Method 'isHidden'
        QtMocHelpers::MethodData<bool(const QModelIndex &) const>(30, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Method 'isGroupHidden'
        QtMocHelpers::MethodData<bool(const GroupType) const>(31, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Method 'isGroupHidden'
        QtMocHelpers::MethodData<bool(const QModelIndex &) const>(31, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Method 'isDevice'
        QtMocHelpers::MethodData<bool(const QModelIndex &) const>(34, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Method 'groupType'
        QtMocHelpers::MethodData<enum GroupType(const QModelIndex &) const>(35, 2, QMC::AccessPublic, 0x80000000 | 32, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Method 'groupIndexes'
        QtMocHelpers::MethodData<QModelIndexList(const GroupType) const>(36, 2, QMC::AccessPublic, 0x80000000 | 37, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Method 'teardownActionForIndex'
        QtMocHelpers::MethodData<QAction *(const QModelIndex &) const>(38, 2, QMC::AccessPublic, 0x80000000 | 39, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Method 'openRecentOption'
        QtMocHelpers::MethodData<QAction *(const QModelIndex &) const>(40, 2, QMC::AccessPublic, 0x80000000 | 39, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Method 'ejectActionForIndex'
        QtMocHelpers::MethodData<QAction *(const QModelIndex &) const>(41, 2, QMC::AccessPublic, 0x80000000 | 39, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Method 'partitionActionForIndex'
        QtMocHelpers::MethodData<QAction *(const QModelIndex &) const>(42, 2, QMC::AccessPublic, 0x80000000 | 39, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Method 'requestTeardown'
        QtMocHelpers::MethodData<void(const QModelIndex &)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Method 'requestEject'
        QtMocHelpers::MethodData<void(const QModelIndex &)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Method 'requestSetup'
        QtMocHelpers::MethodData<void(const QModelIndex &)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Method 'addPlace'
        QtMocHelpers::MethodData<void(const QString &, const QUrl &, const QString &, const QString &)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 29 }, { 0x80000000 | 20, 19 }, { QMetaType::QString, 47 }, { QMetaType::QString, 48 },
        }}),
        // Method 'addPlace'
        QtMocHelpers::MethodData<void(const QString &, const QUrl &, const QString &)>(46, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 29 }, { 0x80000000 | 20, 19 }, { QMetaType::QString, 47 },
        }}),
        // Method 'addPlace'
        QtMocHelpers::MethodData<void(const QString &, const QUrl &)>(46, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 29 }, { 0x80000000 | 20, 19 },
        }}),
        // Method 'addPlace'
        QtMocHelpers::MethodData<void(const QString &, const QUrl &, const QString &, const QString &, const QModelIndex &)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 29 }, { 0x80000000 | 20, 19 }, { QMetaType::QString, 47 }, { QMetaType::QString, 48 },
            { 0x80000000 | 5, 49 },
        }}),
        // Method 'editPlace'
        QtMocHelpers::MethodData<void(const QModelIndex &, const QString &, const QUrl &, const QString &, const QString &)>(50, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { QMetaType::QString, 29 }, { 0x80000000 | 20, 19 }, { QMetaType::QString, 47 },
            { QMetaType::QString, 48 },
        }}),
        // Method 'editPlace'
        QtMocHelpers::MethodData<void(const QModelIndex &, const QString &, const QUrl &, const QString &)>(50, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { QMetaType::QString, 29 }, { 0x80000000 | 20, 19 }, { QMetaType::QString, 47 },
        }}),
        // Method 'editPlace'
        QtMocHelpers::MethodData<void(const QModelIndex &, const QString &, const QUrl &)>(50, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { QMetaType::QString, 29 }, { 0x80000000 | 20, 19 },
        }}),
        // Method 'removePlace'
        QtMocHelpers::MethodData<void(const QModelIndex &) const>(51, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Method 'setPlaceHidden'
        QtMocHelpers::MethodData<void(const QModelIndex &, bool)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { QMetaType::Bool, 16 },
        }}),
        // Method 'setGroupHidden'
        QtMocHelpers::MethodData<void(const GroupType, bool)>(53, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 32, 33 }, { QMetaType::Bool, 16 },
        }}),
        // Method 'movePlace'
        QtMocHelpers::MethodData<bool(int, int)>(54, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Int, 55 }, { QMetaType::Int, 56 },
        }}),
        // Method 'hiddenCount'
        QtMocHelpers::MethodData<int() const>(57, 2, QMC::AccessPublic, QMetaType::Int),
        // Method 'refresh'
        QtMocHelpers::MethodData<void() const>(58, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'supportedSchemes'
        QtMocHelpers::PropertyData<QStringList>(59, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'GroupType'
        QtMocHelpers::EnumData<enum GroupType>(32, 32, QMC::EnumFlags{}).add({
            {   60, GroupType::PlacesType },
            {   61, GroupType::RemoteType },
            {   62, GroupType::RecentlySavedType },
            {   63, GroupType::SearchForType },
            {   64, GroupType::DevicesType },
            {   65, GroupType::RemovableDevicesType },
            {   66, GroupType::UnknownType },
            {   67, GroupType::TagsType },
        }),
        // enum 'DeviceAccessibility'
        QtMocHelpers::EnumData<enum DeviceAccessibility>(68, 68, QMC::EnumFlags{}).add({
            {   69, DeviceAccessibility::SetupNeeded },
            {   70, DeviceAccessibility::SetupInProgress },
            {   71, DeviceAccessibility::Accessible },
            {   72, DeviceAccessibility::TeardownInProgress },
        }),
    };
    return QtMocHelpers::metaObjectData<KFilePlacesModel, qt_meta_tag_ZN16KFilePlacesModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KFilePlacesModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractItemModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16KFilePlacesModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16KFilePlacesModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16KFilePlacesModelE_t>.metaTypes,
    nullptr
} };

void KFilePlacesModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KFilePlacesModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->errorMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->setupDone((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 2: _t->teardownDone((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Solid::ErrorType>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[3]))); break;
        case 3: _t->groupHiddenChanged((*reinterpret_cast<std::add_pointer_t<KFilePlacesModel::GroupType>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 4: _t->reloaded(); break;
        case 5: _t->supportedSchemesChanged(); break;
        case 6: { QUrl _r = _t->url((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QUrl*>(_a[0]) = std::move(_r); }  break;
        case 7: { bool _r = _t->setupNeeded((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 8: { bool _r = _t->isTeardownAllowed((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 9: { bool _r = _t->isEjectAllowed((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 10: { bool _r = _t->isTeardownOverlayRecommended((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 11: { KFilePlacesModel::DeviceAccessibility _r = _t->deviceAccessibility((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])));
            if (_a[0]) *reinterpret_cast<KFilePlacesModel::DeviceAccessibility*>(_a[0]) = std::move(_r); }  break;
        case 12: { QIcon _r = _t->icon((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QIcon*>(_a[0]) = std::move(_r); }  break;
        case 13: { QString _r = _t->text((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 14: { bool _r = _t->isHidden((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 15: { bool _r = _t->isGroupHidden((*reinterpret_cast<std::add_pointer_t<enum GroupType>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 16: { bool _r = _t->isGroupHidden((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 17: { bool _r = _t->isDevice((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 18: { enum GroupType _r = _t->groupType((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])));
            if (_a[0]) *reinterpret_cast<enum GroupType*>(_a[0]) = std::move(_r); }  break;
        case 19: { QModelIndexList _r = _t->groupIndexes((*reinterpret_cast<std::add_pointer_t<enum GroupType>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QModelIndexList*>(_a[0]) = std::move(_r); }  break;
        case 20: { QAction* _r = _t->teardownActionForIndex((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QAction**>(_a[0]) = std::move(_r); }  break;
        case 21: { QAction* _r = _t->openRecentOption((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QAction**>(_a[0]) = std::move(_r); }  break;
        case 22: { QAction* _r = _t->ejectActionForIndex((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QAction**>(_a[0]) = std::move(_r); }  break;
        case 23: { QAction* _r = _t->partitionActionForIndex((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QAction**>(_a[0]) = std::move(_r); }  break;
        case 24: _t->requestTeardown((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 25: _t->requestEject((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 26: _t->requestSetup((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 27: _t->addPlace((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 28: _t->addPlace((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 29: _t->addPlace((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2]))); break;
        case 30: _t->addPlace((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[5]))); break;
        case 31: _t->editPlace((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5]))); break;
        case 32: _t->editPlace((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 33: _t->editPlace((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[3]))); break;
        case 34: _t->removePlace((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 35: _t->setPlaceHidden((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 36: _t->setGroupHidden((*reinterpret_cast<std::add_pointer_t<enum GroupType>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 37: { bool _r = _t->movePlace((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 38: { int _r = _t->hiddenCount();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 39: _t->refresh(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Solid::ErrorType >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KFilePlacesModel::*)(const QString & )>(_a, &KFilePlacesModel::errorMessage, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KFilePlacesModel::*)(const QModelIndex & , bool )>(_a, &KFilePlacesModel::setupDone, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KFilePlacesModel::*)(const QModelIndex & , Solid::ErrorType , const QVariant & )>(_a, &KFilePlacesModel::teardownDone, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KFilePlacesModel::*)(KFilePlacesModel::GroupType , bool )>(_a, &KFilePlacesModel::groupHiddenChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KFilePlacesModel::*)()>(_a, &KFilePlacesModel::reloaded, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (KFilePlacesModel::*)()>(_a, &KFilePlacesModel::supportedSchemesChanged, 5))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QStringList*>(_v) = _t->supportedSchemes(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSupportedSchemes(*reinterpret_cast<QStringList*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *KFilePlacesModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KFilePlacesModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16KFilePlacesModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractItemModel::qt_metacast(_clname);
}

int KFilePlacesModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractItemModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 40)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 40;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 40)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 40;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void KFilePlacesModel::errorMessage(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KFilePlacesModel::setupDone(const QModelIndex & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void KFilePlacesModel::teardownDone(const QModelIndex & _t1, Solid::ErrorType _t2, const QVariant & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}

// SIGNAL 3
void KFilePlacesModel::groupHiddenChanged(KFilePlacesModel::GroupType _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void KFilePlacesModel::reloaded()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void KFilePlacesModel::supportedSchemesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
