/****************************************************************************
** Meta object code from reading C++ file 'kurlnavigator.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kio-6.29.0/src/filewidgets/kurlnavigator.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kurlnavigator.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13KUrlNavigatorE_t {};
} // unnamed namespace

template <> constexpr inline auto KUrlNavigator::qt_create_metaobjectdata<qt_meta_tag_ZN13KUrlNavigatorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KUrlNavigator",
        "activated",
        "",
        "urlChanged",
        "QUrl",
        "url",
        "urlAboutToBeChanged",
        "newUrl",
        "editableStateChanged",
        "editable",
        "historyChanged",
        "urlsDropped",
        "destination",
        "QDropEvent*",
        "event",
        "returnPressed",
        "tabRequested",
        "activeTabRequested",
        "newWindowRequested",
        "urlSelectionRequested",
        "layoutChanged",
        "setLocationUrl",
        "requestActivation",
        "setFocus"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'activated'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'urlChanged'
        QtMocHelpers::SignalData<void(const QUrl &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'urlAboutToBeChanged'
        QtMocHelpers::SignalData<void(const QUrl &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 7 },
        }}),
        // Signal 'editableStateChanged'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'historyChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'urlsDropped'
        QtMocHelpers::SignalData<void(const QUrl &, QDropEvent *)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 12 }, { 0x80000000 | 13, 14 },
        }}),
        // Signal 'returnPressed'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'tabRequested'
        QtMocHelpers::SignalData<void(const QUrl &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'activeTabRequested'
        QtMocHelpers::SignalData<void(const QUrl &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'newWindowRequested'
        QtMocHelpers::SignalData<void(const QUrl &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'urlSelectionRequested'
        QtMocHelpers::SignalData<void(const QUrl &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'layoutChanged'
        QtMocHelpers::SignalData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setLocationUrl'
        QtMocHelpers::SlotData<void(const QUrl &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'requestActivation'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setFocus'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KUrlNavigator, qt_meta_tag_ZN13KUrlNavigatorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KUrlNavigator::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KUrlNavigatorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KUrlNavigatorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KUrlNavigatorE_t>.metaTypes,
    nullptr
} };

void KUrlNavigator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KUrlNavigator *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->activated(); break;
        case 1: _t->urlChanged((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 2: _t->urlAboutToBeChanged((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 3: _t->editableStateChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->historyChanged(); break;
        case 5: _t->urlsDropped((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDropEvent*>>(_a[2]))); break;
        case 6: _t->returnPressed(); break;
        case 7: _t->tabRequested((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 8: _t->activeTabRequested((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 9: _t->newWindowRequested((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 10: _t->urlSelectionRequested((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 11: _t->layoutChanged(); break;
        case 12: _t->setLocationUrl((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 13: _t->requestActivation(); break;
        case 14: _t->setFocus(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KUrlNavigator::*)()>(_a, &KUrlNavigator::activated, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KUrlNavigator::*)(const QUrl & )>(_a, &KUrlNavigator::urlChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KUrlNavigator::*)(const QUrl & )>(_a, &KUrlNavigator::urlAboutToBeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KUrlNavigator::*)(bool )>(_a, &KUrlNavigator::editableStateChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KUrlNavigator::*)()>(_a, &KUrlNavigator::historyChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (KUrlNavigator::*)(const QUrl & , QDropEvent * )>(_a, &KUrlNavigator::urlsDropped, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (KUrlNavigator::*)()>(_a, &KUrlNavigator::returnPressed, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (KUrlNavigator::*)(const QUrl & )>(_a, &KUrlNavigator::tabRequested, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (KUrlNavigator::*)(const QUrl & )>(_a, &KUrlNavigator::activeTabRequested, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (KUrlNavigator::*)(const QUrl & )>(_a, &KUrlNavigator::newWindowRequested, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (KUrlNavigator::*)(const QUrl & )>(_a, &KUrlNavigator::urlSelectionRequested, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (KUrlNavigator::*)()>(_a, &KUrlNavigator::layoutChanged, 11))
            return;
    }
}

const QMetaObject *KUrlNavigator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KUrlNavigator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KUrlNavigatorE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int KUrlNavigator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void KUrlNavigator::activated()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KUrlNavigator::urlChanged(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KUrlNavigator::urlAboutToBeChanged(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KUrlNavigator::editableStateChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void KUrlNavigator::historyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void KUrlNavigator::urlsDropped(const QUrl & _t1, QDropEvent * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void KUrlNavigator::returnPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void KUrlNavigator::tabRequested(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void KUrlNavigator::activeTabRequested(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void KUrlNavigator::newWindowRequested(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void KUrlNavigator::urlSelectionRequested(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void KUrlNavigator::layoutChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}
QT_WARNING_POP
