/****************************************************************************
** Meta object code from reading C++ file 'kcoredirlister.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kio-6.29.0/src/core/kcoredirlister.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kcoredirlister.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14KCoreDirListerE_t {};
} // unnamed namespace

template <> constexpr inline auto KCoreDirLister::qt_create_metaobjectdata<qt_meta_tag_ZN14KCoreDirListerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KCoreDirLister",
        "started",
        "",
        "QUrl",
        "dirUrl",
        "completed",
        "listingDirCompleted",
        "canceled",
        "listingDirCanceled",
        "redirection",
        "oldUrl",
        "newUrl",
        "clear",
        "clearDir",
        "newItems",
        "KFileItemList",
        "items",
        "itemsAdded",
        "directoryUrl",
        "itemsFilteredByMime",
        "itemsDeleted",
        "refreshItems",
        "QList<std::pair<KFileItem,KFileItem>>",
        "infoMessage",
        "msg",
        "percent",
        "totalSize",
        "KIO::filesize_t",
        "size",
        "processedSize",
        "speed",
        "bytes_per_second",
        "jobError",
        "KIO::Job*",
        "job",
        "autoUpdate",
        "showHiddenFiles",
        "dirOnlyMode",
        "delayedMimeTypes",
        "requestMimeTypeWhileListing",
        "nameFilter",
        "mimeFilter",
        "autoErrorHandlingEnabled"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'started'
        QtMocHelpers::SignalData<void(const QUrl &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'completed'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'listingDirCompleted'
        QtMocHelpers::SignalData<void(const QUrl &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'canceled'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'listingDirCanceled'
        QtMocHelpers::SignalData<void(const QUrl &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'redirection'
        QtMocHelpers::SignalData<void(const QUrl &, const QUrl &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 10 }, { 0x80000000 | 3, 11 },
        }}),
        // Signal 'clear'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'clearDir'
        QtMocHelpers::SignalData<void(const QUrl &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'newItems'
        QtMocHelpers::SignalData<void(const KFileItemList &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'itemsAdded'
        QtMocHelpers::SignalData<void(const QUrl &, const KFileItemList &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 18 }, { 0x80000000 | 15, 16 },
        }}),
        // Signal 'itemsFilteredByMime'
        QtMocHelpers::SignalData<void(const KFileItemList &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'itemsDeleted'
        QtMocHelpers::SignalData<void(const KFileItemList &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'refreshItems'
        QtMocHelpers::SignalData<void(const QList<QPair<KFileItem,KFileItem>> &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 16 },
        }}),
        // Signal 'infoMessage'
        QtMocHelpers::SignalData<void(const QString &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 24 },
        }}),
        // Signal 'percent'
        QtMocHelpers::SignalData<void(int)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 25 },
        }}),
        // Signal 'totalSize'
        QtMocHelpers::SignalData<void(KIO::filesize_t)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Signal 'processedSize'
        QtMocHelpers::SignalData<void(KIO::filesize_t)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Signal 'speed'
        QtMocHelpers::SignalData<void(int)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 31 },
        }}),
        // Signal 'jobError'
        QtMocHelpers::SignalData<void(KIO::Job *)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 33, 34 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'autoUpdate'
        QtMocHelpers::PropertyData<bool>(35, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'showHiddenFiles'
        QtMocHelpers::PropertyData<bool>(36, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'dirOnlyMode'
        QtMocHelpers::PropertyData<bool>(37, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'delayedMimeTypes'
        QtMocHelpers::PropertyData<bool>(38, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'requestMimeTypeWhileListing'
        QtMocHelpers::PropertyData<bool>(39, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'nameFilter'
        QtMocHelpers::PropertyData<QString>(40, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'mimeFilter'
        QtMocHelpers::PropertyData<QStringList>(41, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet),
        // property 'autoErrorHandlingEnabled'
        QtMocHelpers::PropertyData<bool>(42, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KCoreDirLister, qt_meta_tag_ZN14KCoreDirListerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KCoreDirLister::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14KCoreDirListerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14KCoreDirListerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14KCoreDirListerE_t>.metaTypes,
    nullptr
} };

void KCoreDirLister::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KCoreDirLister *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->started((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 1: _t->completed(); break;
        case 2: _t->listingDirCompleted((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 3: _t->canceled(); break;
        case 4: _t->listingDirCanceled((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 5: _t->redirection((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2]))); break;
        case 6: _t->clear(); break;
        case 7: _t->clearDir((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 8: _t->newItems((*reinterpret_cast<std::add_pointer_t<KFileItemList>>(_a[1]))); break;
        case 9: _t->itemsAdded((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KFileItemList>>(_a[2]))); break;
        case 10: _t->itemsFilteredByMime((*reinterpret_cast<std::add_pointer_t<KFileItemList>>(_a[1]))); break;
        case 11: _t->itemsDeleted((*reinterpret_cast<std::add_pointer_t<KFileItemList>>(_a[1]))); break;
        case 12: _t->refreshItems((*reinterpret_cast<std::add_pointer_t<QList<std::pair<KFileItem,KFileItem>>>>(_a[1]))); break;
        case 13: _t->infoMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 14: _t->percent((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 15: _t->totalSize((*reinterpret_cast<std::add_pointer_t<KIO::filesize_t>>(_a[1]))); break;
        case 16: _t->processedSize((*reinterpret_cast<std::add_pointer_t<KIO::filesize_t>>(_a[1]))); break;
        case 17: _t->speed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 18: _t->jobError((*reinterpret_cast<std::add_pointer_t<KIO::Job*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KCoreDirLister::*)(const QUrl & )>(_a, &KCoreDirLister::started, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCoreDirLister::*)()>(_a, &KCoreDirLister::completed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCoreDirLister::*)(const QUrl & )>(_a, &KCoreDirLister::listingDirCompleted, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCoreDirLister::*)()>(_a, &KCoreDirLister::canceled, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCoreDirLister::*)(const QUrl & )>(_a, &KCoreDirLister::listingDirCanceled, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCoreDirLister::*)(const QUrl & , const QUrl & )>(_a, &KCoreDirLister::redirection, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCoreDirLister::*)()>(_a, &KCoreDirLister::clear, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCoreDirLister::*)(const QUrl & )>(_a, &KCoreDirLister::clearDir, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCoreDirLister::*)(const KFileItemList & )>(_a, &KCoreDirLister::newItems, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCoreDirLister::*)(const QUrl & , const KFileItemList & )>(_a, &KCoreDirLister::itemsAdded, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCoreDirLister::*)(const KFileItemList & )>(_a, &KCoreDirLister::itemsFilteredByMime, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCoreDirLister::*)(const KFileItemList & )>(_a, &KCoreDirLister::itemsDeleted, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCoreDirLister::*)(const QList<QPair<KFileItem,KFileItem>> & )>(_a, &KCoreDirLister::refreshItems, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCoreDirLister::*)(const QString & )>(_a, &KCoreDirLister::infoMessage, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCoreDirLister::*)(int )>(_a, &KCoreDirLister::percent, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCoreDirLister::*)(KIO::filesize_t )>(_a, &KCoreDirLister::totalSize, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCoreDirLister::*)(KIO::filesize_t )>(_a, &KCoreDirLister::processedSize, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCoreDirLister::*)(int )>(_a, &KCoreDirLister::speed, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCoreDirLister::*)(KIO::Job * )>(_a, &KCoreDirLister::jobError, 18))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->autoUpdate(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->showHiddenFiles(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->dirOnlyMode(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->delayedMimeTypes(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->requestMimeTypeWhileListing(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->nameFilter(); break;
        case 6: *reinterpret_cast<QStringList*>(_v) = _t->mimeFilters(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->autoErrorHandlingEnabled(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setAutoUpdate(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setShowHiddenFiles(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setDirOnlyMode(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setDelayedMimeTypes(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setRequestMimeTypeWhileListing(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setNameFilter(*reinterpret_cast<QString*>(_v)); break;
        case 6: _t->setMimeFilter(*reinterpret_cast<QStringList*>(_v)); break;
        case 7: _t->setAutoErrorHandlingEnabled(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::ResetProperty) {
        switch (_id) {
        case 6: _t->clearMimeFilter(); break;
        default: break;
        }
    }
}

const QMetaObject *KCoreDirLister::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KCoreDirLister::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14KCoreDirListerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KCoreDirLister::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 19;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void KCoreDirLister::started(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KCoreDirLister::completed()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KCoreDirLister::listingDirCompleted(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KCoreDirLister::canceled()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void KCoreDirLister::listingDirCanceled(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void KCoreDirLister::redirection(const QUrl & _t1, const QUrl & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void KCoreDirLister::clear()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void KCoreDirLister::clearDir(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void KCoreDirLister::newItems(const KFileItemList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void KCoreDirLister::itemsAdded(const QUrl & _t1, const KFileItemList & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2);
}

// SIGNAL 10
void KCoreDirLister::itemsFilteredByMime(const KFileItemList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void KCoreDirLister::itemsDeleted(const KFileItemList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void KCoreDirLister::refreshItems(const QList<QPair<KFileItem,KFileItem>> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void KCoreDirLister::infoMessage(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void KCoreDirLister::percent(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void KCoreDirLister::totalSize(KIO::filesize_t _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void KCoreDirLister::processedSize(KIO::filesize_t _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void KCoreDirLister::speed(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void KCoreDirLister::jobError(KIO::Job * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}
QT_WARNING_POP
