/****************************************************************************
** Meta object code from reading C++ file 'kurlrequester.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kio-6.29.0/src/widgets/kurlrequester.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kurlrequester.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13KUrlRequesterE_t {};
} // unnamed namespace

template <> constexpr inline auto KUrlRequester::qt_create_metaobjectdata<qt_meta_tag_ZN13KUrlRequesterE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KUrlRequester",
        "textChanged",
        "",
        "textEdited",
        "returnPressed",
        "text",
        "openFileDialog",
        "KUrlRequester*",
        "urlSelected",
        "QUrl",
        "setUrl",
        "url",
        "setStartDir",
        "startDir",
        "setText",
        "clear",
        "nameFilters",
        "mode",
        "KFile::Modes",
        "acceptMode",
        "QFileDialog::AcceptMode",
        "placeholderText",
        "fileDialogModality",
        "Qt::WindowModality"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'textChanged'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'textEdited'
        QtMocHelpers::SignalData<void(const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'returnPressed'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'openFileDialog'
        QtMocHelpers::SignalData<void(KUrlRequester *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 2 },
        }}),
        // Signal 'urlSelected'
        QtMocHelpers::SignalData<void(const QUrl &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 2 },
        }}),
        // Slot 'setUrl'
        QtMocHelpers::SlotData<void(const QUrl &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 11 },
        }}),
        // Slot 'setStartDir'
        QtMocHelpers::SlotData<void(const QUrl &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 13 },
        }}),
        // Slot 'setText'
        QtMocHelpers::SlotData<void(const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'clear'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'url'
        QtMocHelpers::PropertyData<QUrl>(11, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::User, 0),
        // property 'nameFilters'
        QtMocHelpers::PropertyData<QStringList>(16, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'mode'
        QtMocHelpers::PropertyData<KFile::Modes>(17, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'acceptMode'
        QtMocHelpers::PropertyData<QFileDialog::AcceptMode>(19, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'placeholderText'
        QtMocHelpers::PropertyData<QString>(21, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'text'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'fileDialogModality'
        QtMocHelpers::PropertyData<Qt::WindowModality>(22, 0x80000000 | 23, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KUrlRequester, qt_meta_tag_ZN13KUrlRequesterE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN13KUrlRequesterE[] = {
    QMetaObject::SuperData::link<KFile::staticMetaObject>(),
    QMetaObject::SuperData::link<QFileDialog::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KUrlRequester::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KUrlRequesterE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KUrlRequesterE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN13KUrlRequesterE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KUrlRequesterE_t>.metaTypes,
    nullptr
} };

void KUrlRequester::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KUrlRequester *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->textChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->textEdited((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->returnPressed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->openFileDialog((*reinterpret_cast<std::add_pointer_t<KUrlRequester*>>(_a[1]))); break;
        case 4: _t->urlSelected((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 5: _t->setUrl((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 6: _t->setStartDir((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 7: _t->setText((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->clear(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KUrlRequester* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KUrlRequester::*)(const QString & )>(_a, &KUrlRequester::textChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KUrlRequester::*)(const QString & )>(_a, &KUrlRequester::textEdited, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KUrlRequester::*)(const QString & )>(_a, &KUrlRequester::returnPressed, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KUrlRequester::*)(KUrlRequester * )>(_a, &KUrlRequester::openFileDialog, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KUrlRequester::*)(const QUrl & )>(_a, &KUrlRequester::urlSelected, 4))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QUrl*>(_v) = _t->url(); break;
        case 1: *reinterpret_cast<QStringList*>(_v) = _t->nameFilters(); break;
        case 2: *reinterpret_cast<KFile::Modes*>(_v) = _t->mode(); break;
        case 3: *reinterpret_cast<QFileDialog::AcceptMode*>(_v) = _t->acceptMode(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->placeholderText(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->text(); break;
        case 6: *reinterpret_cast<Qt::WindowModality*>(_v) = _t->fileDialogModality(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setUrl(*reinterpret_cast<QUrl*>(_v)); break;
        case 1: _t->setNameFilters(*reinterpret_cast<QStringList*>(_v)); break;
        case 2: _t->setMode(*reinterpret_cast<KFile::Modes*>(_v)); break;
        case 3: _t->setAcceptMode(*reinterpret_cast<QFileDialog::AcceptMode*>(_v)); break;
        case 4: _t->setPlaceholderText(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setText(*reinterpret_cast<QString*>(_v)); break;
        case 6: _t->setFileDialogModality(*reinterpret_cast<Qt::WindowModality*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *KUrlRequester::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KUrlRequester::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KUrlRequesterE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int KUrlRequester::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void KUrlRequester::textChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KUrlRequester::textEdited(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KUrlRequester::returnPressed(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KUrlRequester::openFileDialog(KUrlRequester * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void KUrlRequester::urlSelected(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN18KUrlComboRequesterE_t {};
} // unnamed namespace

template <> constexpr inline auto KUrlComboRequester::qt_create_metaobjectdata<qt_meta_tag_ZN18KUrlComboRequesterE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KUrlComboRequester"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KUrlComboRequester, qt_meta_tag_ZN18KUrlComboRequesterE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KUrlComboRequester::staticMetaObject = { {
    QMetaObject::SuperData::link<KUrlRequester::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18KUrlComboRequesterE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18KUrlComboRequesterE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18KUrlComboRequesterE_t>.metaTypes,
    nullptr
} };

void KUrlComboRequester::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KUrlComboRequester *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KUrlComboRequester::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KUrlComboRequester::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18KUrlComboRequesterE_t>.strings))
        return static_cast<void*>(this);
    return KUrlRequester::qt_metacast(_clname);
}

int KUrlComboRequester::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KUrlRequester::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
