/****************************************************************************
** Meta object code from reading C++ file 'kcoredirlister_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kio-6.29.0/src/core/kcoredirlister_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kcoredirlister_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19KCoreDirListerCacheE_t {};
} // unnamed namespace

template <> constexpr inline auto KCoreDirListerCache::qt_create_metaobjectdata<qt_meta_tag_ZN19KCoreDirListerCacheE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KCoreDirListerCache",
        "slotFilesAdded",
        "",
        "urlDirectory",
        "slotFilesRemoved",
        "fileList",
        "slotFilesChanged",
        "slotFileRenamed",
        "srcUrl",
        "dstUrl",
        "dstPath",
        "pruneCachedDirectories",
        "slotFileDirty",
        "_file",
        "slotFileCreated",
        "slotFileDeleted",
        "slotEntries",
        "KIO::Job*",
        "job",
        "KIO::UDSEntryList",
        "entries",
        "slotResult",
        "KJob*",
        "j",
        "slotRedirection",
        "QUrl",
        "url",
        "slotUpdateEntries",
        "slotUpdateResult",
        "processPendingUpdates"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotFilesAdded'
        QtMocHelpers::SlotData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'slotFilesRemoved'
        QtMocHelpers::SlotData<void(const QStringList &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 5 },
        }}),
        // Slot 'slotFilesChanged'
        QtMocHelpers::SlotData<void(const QStringList &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 5 },
        }}),
        // Slot 'slotFileRenamed'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::QString, 10 },
        }}),
        // Slot 'pruneCachedDirectories'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotFileDirty'
        QtMocHelpers::SlotData<void(const QString &)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'slotFileCreated'
        QtMocHelpers::SlotData<void(const QString &)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'slotFileDeleted'
        QtMocHelpers::SlotData<void(const QString &)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'slotEntries'
        QtMocHelpers::SlotData<void(KIO::Job *, const KIO::UDSEntryList &)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { 0x80000000 | 19, 20 },
        }}),
        // Slot 'slotResult'
        QtMocHelpers::SlotData<void(KJob *)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'slotRedirection'
        QtMocHelpers::SlotData<void(KIO::Job *, const QUrl &)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { 0x80000000 | 25, 26 },
        }}),
        // Slot 'slotUpdateEntries'
        QtMocHelpers::SlotData<void(KIO::Job *, const KIO::UDSEntryList &)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { 0x80000000 | 19, 20 },
        }}),
        // Slot 'slotUpdateResult'
        QtMocHelpers::SlotData<void(KJob *)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 22, 18 },
        }}),
        // Slot 'processPendingUpdates'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KCoreDirListerCache, qt_meta_tag_ZN19KCoreDirListerCacheE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KCoreDirListerCache::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19KCoreDirListerCacheE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19KCoreDirListerCacheE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19KCoreDirListerCacheE_t>.metaTypes,
    nullptr
} };

void KCoreDirListerCache::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KCoreDirListerCache *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotFilesAdded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->slotFilesRemoved((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 2: _t->slotFilesChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 3: _t->slotFileRenamed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 4: _t->pruneCachedDirectories(); break;
        case 5: _t->slotFileDirty((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->slotFileCreated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->slotFileDeleted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->slotEntries((*reinterpret_cast<std::add_pointer_t<KIO::Job*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KIO::UDSEntryList>>(_a[2]))); break;
        case 9: _t->slotResult((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        case 10: _t->slotRedirection((*reinterpret_cast<std::add_pointer_t<KIO::Job*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2]))); break;
        case 11: _t->slotUpdateEntries((*reinterpret_cast<std::add_pointer_t<KIO::Job*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KIO::UDSEntryList>>(_a[2]))); break;
        case 12: _t->slotUpdateResult((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        case 13: _t->processPendingUpdates(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KJob* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KJob* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *KCoreDirListerCache::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KCoreDirListerCache::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19KCoreDirListerCacheE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KCoreDirListerCache::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN21KCoreDirListerPrivate14CachedItemsJobE_t {};
} // unnamed namespace

template <> constexpr inline auto KCoreDirListerPrivate::CachedItemsJob::qt_create_metaobjectdata<qt_meta_tag_ZN21KCoreDirListerPrivate14CachedItemsJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KCoreDirListerPrivate::CachedItemsJob",
        "done",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'done'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CachedItemsJob, qt_meta_tag_ZN21KCoreDirListerPrivate14CachedItemsJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KCoreDirListerPrivate::CachedItemsJob::staticMetaObject = { {
    QMetaObject::SuperData::link<KJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21KCoreDirListerPrivate14CachedItemsJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21KCoreDirListerPrivate14CachedItemsJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21KCoreDirListerPrivate14CachedItemsJobE_t>.metaTypes,
    nullptr
} };

void KCoreDirListerPrivate::CachedItemsJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CachedItemsJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->done(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *KCoreDirListerPrivate::CachedItemsJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KCoreDirListerPrivate::CachedItemsJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21KCoreDirListerPrivate14CachedItemsJobE_t>.strings))
        return static_cast<void*>(this);
    return KJob::qt_metacast(_clname);
}

int KCoreDirListerPrivate::CachedItemsJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
QT_WARNING_POP
