/****************************************************************************
** Meta object code from reading C++ file 'kurlnavigatorbutton_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kio-6.29.0/src/filewidgets/kurlnavigatorbutton_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kurlnavigatorbutton_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KDEPrivate19KUrlNavigatorButtonE_t {};
} // unnamed namespace

template <> constexpr inline auto KDEPrivate::KUrlNavigatorButton::qt_create_metaobjectdata<qt_meta_tag_ZN10KDEPrivate19KUrlNavigatorButtonE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KDEPrivate::KUrlNavigatorButton",
        "urlsDroppedOnNavButton",
        "",
        "QUrl",
        "destination",
        "QDropEvent*",
        "event",
        "navigatorButtonActivated",
        "url",
        "Qt::MouseButton",
        "button",
        "Qt::KeyboardModifiers",
        "modifiers",
        "startedTextResolving",
        "finishedTextResolving",
        "requestSubDirs",
        "startSubDirsJob",
        "addEntriesToSubDirs",
        "KIO::Job*",
        "job",
        "KIO::UDSEntryList",
        "entries",
        "openSubDirsMenu",
        "KJob*",
        "replaceButton",
        "slotUrlsDropped",
        "QAction*",
        "action",
        "slotMenuActionClicked",
        "statFinished",
        "plainText"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'urlsDroppedOnNavButton'
        QtMocHelpers::SignalData<void(const QUrl &, QDropEvent *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Signal 'navigatorButtonActivated'
        QtMocHelpers::SignalData<void(const QUrl &, Qt::MouseButton, Qt::KeyboardModifiers)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 8 }, { 0x80000000 | 9, 10 }, { 0x80000000 | 11, 12 },
        }}),
        // Signal 'startedTextResolving'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'finishedTextResolving'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'requestSubDirs'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'startSubDirsJob'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'addEntriesToSubDirs'
        QtMocHelpers::SlotData<void(KIO::Job *, const KIO::UDSEntryList &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 }, { 0x80000000 | 20, 21 },
        }}),
        // Slot 'openSubDirsMenu'
        QtMocHelpers::SlotData<void(KJob *)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 23, 19 },
        }}),
        // Slot 'replaceButton'
        QtMocHelpers::SlotData<void(KJob *)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 23, 19 },
        }}),
        // Slot 'slotUrlsDropped'
        QtMocHelpers::SlotData<void(QAction *, QDropEvent *)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 26, 27 }, { 0x80000000 | 5, 6 },
        }}),
        // Slot 'slotMenuActionClicked'
        QtMocHelpers::SlotData<void(QAction *, Qt::MouseButton)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 26, 27 }, { 0x80000000 | 9, 10 },
        }}),
        // Slot 'statFinished'
        QtMocHelpers::SlotData<void(KJob *)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 23, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'plainText'
        QtMocHelpers::PropertyData<QString>(30, QMetaType::QString, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KUrlNavigatorButton, qt_meta_tag_ZN10KDEPrivate19KUrlNavigatorButtonE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KDEPrivate::KUrlNavigatorButton::staticMetaObject = { {
    QMetaObject::SuperData::link<KUrlNavigatorButtonBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KDEPrivate19KUrlNavigatorButtonE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KDEPrivate19KUrlNavigatorButtonE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KDEPrivate19KUrlNavigatorButtonE_t>.metaTypes,
    nullptr
} };

void KDEPrivate::KUrlNavigatorButton::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KUrlNavigatorButton *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->urlsDroppedOnNavButton((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDropEvent*>>(_a[2]))); break;
        case 1: _t->navigatorButtonActivated((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::MouseButton>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[3]))); break;
        case 2: _t->startedTextResolving(); break;
        case 3: _t->finishedTextResolving(); break;
        case 4: _t->requestSubDirs(); break;
        case 5: _t->startSubDirsJob(); break;
        case 6: _t->addEntriesToSubDirs((*reinterpret_cast<std::add_pointer_t<KIO::Job*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KIO::UDSEntryList>>(_a[2]))); break;
        case 7: _t->openSubDirsMenu((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        case 8: _t->replaceButton((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        case 9: _t->slotUrlsDropped((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDropEvent*>>(_a[2]))); break;
        case 10: _t->slotMenuActionClicked((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::MouseButton>>(_a[2]))); break;
        case 11: _t->statFinished((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KJob* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KJob* >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KJob* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KUrlNavigatorButton::*)(const QUrl & , QDropEvent * )>(_a, &KUrlNavigatorButton::urlsDroppedOnNavButton, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KUrlNavigatorButton::*)(const QUrl & , Qt::MouseButton , Qt::KeyboardModifiers )>(_a, &KUrlNavigatorButton::navigatorButtonActivated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KUrlNavigatorButton::*)()>(_a, &KUrlNavigatorButton::startedTextResolving, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KUrlNavigatorButton::*)()>(_a, &KUrlNavigatorButton::finishedTextResolving, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->plainText(); break;
        default: break;
        }
    }
}

const QMetaObject *KDEPrivate::KUrlNavigatorButton::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDEPrivate::KUrlNavigatorButton::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KDEPrivate19KUrlNavigatorButtonE_t>.strings))
        return static_cast<void*>(this);
    return KUrlNavigatorButtonBase::qt_metacast(_clname);
}

int KDEPrivate::KUrlNavigatorButton::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KUrlNavigatorButtonBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void KDEPrivate::KUrlNavigatorButton::urlsDroppedOnNavButton(const QUrl & _t1, QDropEvent * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void KDEPrivate::KUrlNavigatorButton::navigatorButtonActivated(const QUrl & _t1, Qt::MouseButton _t2, Qt::KeyboardModifiers _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 2
void KDEPrivate::KUrlNavigatorButton::startedTextResolving()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void KDEPrivate::KUrlNavigatorButton::finishedTextResolving()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
