/****************************************************************************
** Meta object code from reading C++ file 'fileundomanager_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kio-6.29.0/src/widgets/fileundomanager_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fileundomanager_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3KIO15CommandRecorderE_t {};
} // unnamed namespace

template <> constexpr inline auto KIO::CommandRecorder::qt_create_metaobjectdata<qt_meta_tag_ZN3KIO15CommandRecorderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KIO::CommandRecorder",
        "slotResult",
        "",
        "KJob*",
        "job",
        "slotCopyingDone",
        "KIO::Job*",
        "QUrl",
        "from",
        "to",
        "directory",
        "renamed",
        "slotCopyingLinkDone",
        "target",
        "slotDirectoryCreated",
        "url",
        "slotBatchRenamingDone"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotResult'
        QtMocHelpers::SlotData<void(KJob *)>(1, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotCopyingDone'
        QtMocHelpers::SlotData<void(KIO::Job *, const QUrl &, const QUrl &, const QDateTime &, bool, bool)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 6, 2 }, { 0x80000000 | 7, 8 }, { 0x80000000 | 7, 9 }, { QMetaType::QDateTime, 2 },
            { QMetaType::Bool, 10 }, { QMetaType::Bool, 11 },
        }}),
        // Slot 'slotCopyingLinkDone'
        QtMocHelpers::SlotData<void(KIO::Job *, const QUrl &, const QString &, const QUrl &)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 6, 2 }, { 0x80000000 | 7, 8 }, { QMetaType::QString, 13 }, { 0x80000000 | 7, 9 },
        }}),
        // Slot 'slotDirectoryCreated'
        QtMocHelpers::SlotData<void(const QUrl &)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 15 },
        }}),
        // Slot 'slotBatchRenamingDone'
        QtMocHelpers::SlotData<void(const QUrl &, const QUrl &)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 8 }, { 0x80000000 | 7, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CommandRecorder, qt_meta_tag_ZN3KIO15CommandRecorderE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KIO::CommandRecorder::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KIO15CommandRecorderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KIO15CommandRecorderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3KIO15CommandRecorderE_t>.metaTypes,
    nullptr
} };

void KIO::CommandRecorder::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CommandRecorder *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotResult((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        case 1: _t->slotCopyingDone((*reinterpret_cast<std::add_pointer_t<KIO::Job*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6]))); break;
        case 2: _t->slotCopyingLinkDone((*reinterpret_cast<std::add_pointer_t<KIO::Job*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[4]))); break;
        case 3: _t->slotDirectoryCreated((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 4: _t->slotBatchRenamingDone((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2]))); break;
        default: ;
        }
    }
}

const QMetaObject *KIO::CommandRecorder::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KIO::CommandRecorder::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KIO15CommandRecorderE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KIO::CommandRecorder::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN3KIO22FileUndoManagerPrivateE_t {};
} // unnamed namespace

template <> constexpr inline auto KIO::FileUndoManagerPrivate::qt_create_metaobjectdata<qt_meta_tag_ZN3KIO22FileUndoManagerPrivateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KIO::FileUndoManagerPrivate",
        "push",
        "",
        "command",
        "pop",
        "lock",
        "unlock",
        "slotPushUndoCommand",
        "slotPopUndoCommand",
        "slotLock",
        "slotUnlock",
        "processStep",
        "redo",
        "slotUndoResult",
        "KJob*",
        "slotRedoResult"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'push'
        QtMocHelpers::SignalData<void(const QByteArray &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 3 },
        }}),
        // Signal 'pop'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'lock'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'unlock'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotPushUndoCommand'
        QtMocHelpers::SlotData<void(QByteArray)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 2 },
        }}),
        // Slot 'slotPopUndoCommand'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotLock'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotUnlock'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'processStep'
        QtMocHelpers::SlotData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 12 },
        }}),
        // Slot 'slotUndoResult'
        QtMocHelpers::SlotData<void(KJob *)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 2 },
        }}),
        // Slot 'slotRedoResult'
        QtMocHelpers::SlotData<void(KJob *)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FileUndoManagerPrivate, qt_meta_tag_ZN3KIO22FileUndoManagerPrivateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KIO::FileUndoManagerPrivate::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KIO22FileUndoManagerPrivateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KIO22FileUndoManagerPrivateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3KIO22FileUndoManagerPrivateE_t>.metaTypes,
    nullptr
} };

void KIO::FileUndoManagerPrivate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FileUndoManagerPrivate *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->push((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 1: _t->pop(); break;
        case 2: _t->lock(); break;
        case 3: _t->unlock(); break;
        case 4: _t->slotPushUndoCommand((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 5: _t->slotPopUndoCommand(); break;
        case 6: _t->slotLock(); break;
        case 7: _t->slotUnlock(); break;
        case 8: _t->processStep((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->slotUndoResult((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        case 10: _t->slotRedoResult((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FileUndoManagerPrivate::*)(const QByteArray & )>(_a, &FileUndoManagerPrivate::push, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileUndoManagerPrivate::*)()>(_a, &FileUndoManagerPrivate::pop, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileUndoManagerPrivate::*)()>(_a, &FileUndoManagerPrivate::lock, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileUndoManagerPrivate::*)()>(_a, &FileUndoManagerPrivate::unlock, 3))
            return;
    }
}

const QMetaObject *KIO::FileUndoManagerPrivate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KIO::FileUndoManagerPrivate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KIO22FileUndoManagerPrivateE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KIO::FileUndoManagerPrivate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void KIO::FileUndoManagerPrivate::push(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KIO::FileUndoManagerPrivate::pop()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KIO::FileUndoManagerPrivate::lock()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void KIO::FileUndoManagerPrivate::unlock()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
